//! Parser for the oyster shell, the grammar is outlined below, `extras` are allowed everywhere,
//! *except* inside `word`.
//! `_` means that the rule does not produce an explicit node.
//!
//! ```grammar
//! program     ::= (statement terminator)*
//! _statement  ::= pipeline
//! _terminator ::= SEMICOLON | NEWLINES | EOF
//! pipeline    ::= command (PIPE NEWLINES? command)*
//! command     ::= word+
//! word        ::= (PLAIN_TEXT|DQUOTES PLAIN_TEXT DQUOTES)+
//!
//! extras      ::= COMMENT | WHITESPACE | BACKSLASH_N
//! ```

pub mod ast;
mod cst;
mod lexer;
mod parser;

pub use cst::ParseTree;
pub use lexer::{Lexer, Token, TokenKind};
pub use parser::{NodeKind, ParseError, ParseEvent, Parser};
